------------------------------------------
-- Setting up libraries
------------------------------------------

local AddonName = "iWait"
iWait = LibStub("AceAddon-3.0"):NewAddon(AddonName, "AceEvent-3.0", "AceTimer-3.0");

local QTip = LibStub("LibQTip-1.0");
--local L = LibStub("AceLocale-3.0"):GetLocale(AddonName);

local Tooltip;
local RosterTimer;

iWait.Broker = LibStub("LibDataBroker-1.1"):NewDataObject(AddonName, {
	type = "data source",
	text = "",

	OnClick = function(anchor, button)
		if( not iWait.Queued ) then
		
			if( button == "LeftButton" ) then
				ToggleLFDParentFrame();
			elseif( button == "MiddleButton" ) then
				ToggleFrame(RaidParentFrame); -- ToggleLFRParentFrame() doesn't seem to work
			elseif( button == "RightButton" ) then
				TogglePVPFrame();
			end
			
		else
		
			if( iWait.IsPvP ) then
				if( button == "LeftButton" and iWait.PvPState == 3 ) then
					ToggleWorldStateScoreFrame();
				elseif( button == "RightButton" ) then
					ToggleDropDownMenu(1, nil, MiniMapBattlefieldDropDown, anchor, 0, 0);
				end
			else
				if( button == "RightButton") then
					ToggleDropDownMenu(1, nil, MiniMapLFGFrameDropDown, anchor, 0, 0);
				end
			end
			
		end
	end,

	OnEnter = function(anchor)
		if( not iWait.Queued ) then
			return;
		end
		
		if( iWait.IsPvP ) then
			-- no PvP tooltip atm
		else
			LFGSearchStatus:SetParent(anchor);
			LFGSearchStatus:SetClampedToScreen(true);
			LFGSearchStatus:SetPoint("TOPRIGHT", anchor, "BOTTOM", 175, 0);
			LFGSearchStatus:Show();
		end
	end,
	
	OnLeave = function()
		if( iWait.IsPvP ) then
			-- no PvP tooltip atm
		else
			LFGSearchStatus:Hide();
		end
	end,
});

------------------------------------------
-- Variables, functions and colors
------------------------------------------

local COLOR_GOLD = "|cfffed100%s|r";
local COLOR_GRAY = "|cffaaaaaa%s|r";
local COLOR_RED  = "|cffff0000%s|r";
local COLOR_GREEN= "|cff00ff00%s|r";
local COLOR_YELLOW= "|cffffff00%s|r"; --Interface\BattlefieldFrame\UI-Battlefield-Icon

iWait.Queued = false;
iWait.IsRaid = false; -- determines whether we queued for normal dungeon (lfd) or raid (lfr)
iWait.IsPvP = false;
iWait.IsArena = false;

iWait.PvPState = 0;
local PvPStates = {
	active = 3,
	confirm = 2,
	queued = 1,
	none = 0,
	error = 0,
};

local function StatusColor(text, need, current)
	if( current < need ) then
		return (COLOR_GREEN):format(text);
	else
		return (COLOR_RED):format(text);
	end
end

local function Status(tank, heal, dps)
	if( iWait.IsRaid ) then
		return ("LFR: %s%s%s"):format(
			StatusColor("T", 1, tank),
			StatusColor("H", 1, heal),
			StatusColor("D", 1, dps)
		);
	else
		return ("LFD: %s%s%s%s%s"):format(
			StatusColor("T", 1, tank),
			StatusColor("H", 1, heal),
			StatusColor("D", 3, dps),
			StatusColor("D", 2, dps),
			StatusColor("D", 1, dps)
		);
	end
end

local function BrokerIcon(icon)
	iWait.Broker.icon = "Interface\\Addons\\iWait\\"..icon;
end

------------------------------------------
-- OnInitialize and ResetBroker
------------------------------------------

function iWait:OnInitialize()
	self:ResetBroker();
	
	self:RegisterEvent("LFG_UPDATE", "LFGWait");
	self:RegisterEvent("LFG_QUEUE_STATUS_UPDATE", "LFGWait");
	self:RegisterEvent("LFG_PROPOSAL_UPDATE", "LFGWait");
	self:RegisterEvent("LFG_PROPOSAL_SUCCEEDED", "LFGWait");
	
	self:RegisterEvent("UPDATE_BATTLEFIELD_STATUS", "PvPWait");
end

function iWait:ResetBroker()
	self.Broker.text = "";
	BrokerIcon("LFD-Gray");
	self.Queued = false;
	self.IsPvP = false;
	self.IsRaid = false;
	self.IsArena = false;
	self.PvPState = 0;
end

------------------------------------------
-- EventHandler
------------------------------------------

function iWait:PvPWait()
	local state = 0;
	
	for i = 1, MAX_WORLD_PVP_QUEUES do
		local status, _, _, _, _, teamSize, _ = GetBattlefieldStatus(i);
		if( PvPStates[status] > state ) then
			state = PvPStates[status];
		end
		
		if( teamSize ~= 0 ) then
			self.IsArena = true;
		end
	end
	
	self.PvPState = state;
	
	if( self.PvPState == PvPStates["active"] ) then
		self.Queued = false;
		BrokerIcon("LFD-Green");
		self.Broker.text = (COLOR_GREEN):format("In Progress");
	elseif( self.PvPState == PvPStates["confirm"] ) then
		self.Queued = false;
		BrokerIcon("LFD-Green");
		
		if( self.IsArena ) then
			self.Broker.text = (COLOR_YELLOW):format("Enter Arena");
		else
			self.Broker.text = (COLOR_YELLOW):format("Enter Battlefield");
		end
	elseif( self.PvPState == PvPStates["queued"] ) then
		self.Queued = true;
		self.IsPvP = true;
		BrokerIcon("LFD-Green");
		
		if( self.IsArena ) then
			self.Broker.text = (COLOR_YELLOW):format("Arena");
		else
			self.Broker.text = (COLOR_YELLOW):format("Battlefield");
		end
	else
		self:ResetBroker();
	end
end

function iWait:LFGWait(event)
	local mode, submode = GetLFGMode();
	
	if( not mode or mode == "suspended" ) then
		self:ResetBroker();
		return;
	elseif( mode == "lfgparty" ) then
		self.Broker.text = (COLOR_GREEN):format("In Progress");
		BrokerIcon("LFD-Green");
		self.Queued = false;
		return;
	elseif( mode == "proposal" ) then
		if( submode and submode == "unaccepted" ) then
			self.Broker.text = (COLOR_GREEN):format("Enter Dungeon");
			BrokerIcon("LFD-Green");
		else
			self.Broker.text = (COLOR_YELLOW):format("Waiting...");
		end
	end
	
	local hasData, _, tankNeed, healNeed, dpsNeed, iniType, _, _, _, _, _, _, _ = GetLFGQueueStats();
	
	if( mode == "queued" ) then
		self.Queued = true;
		BrokerIcon("LFD-Green");
		
		if( hasData ) then
			if( iniType and iniType == 2 ) then
				self.IsRaid = true;
			end
			
			self.Broker.text = Status(tankNeed, healNeed, dpsNeed);
		else
			self.Broker.text = (COLOR_YELLOW):format("Preparing...");
		end
	end
	
	if( not mode ) then mode = "" end
	if( not submode ) then submode = "" end
	--print(event.." -- "..mode.." "..submode);
	if( not iniType ) then iniType = "nil" end
	--print(iniType);
end

------------------------------------------
-- Hiding icons
------------------------------------------

MiniMapLFGFrame:SetScript("OnShow", function() MiniMapLFGFrame:Hide() end);
MiniMapBattlefieldFrame:SetScript("OnShow", function() MiniMapBattlefieldFrame:Hide() end);